/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the "License").  You may not use this file except
 * in compliance with the License.
 *
 * You can obtain a copy of the license at
 * glassfish/bootstrap/legal/CDDLv1.0.txt or
 * https://glassfish.dev.java.net/public/CDDLv1.0.html.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * HEADER in each file and include the License file at
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
 * add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your
 * own identifying information: Portions Copyright [yyyy]
 * [name of copyright owner]
 */

package com.sun.grizzly.cometd;

import com.sun.enterprise.web.connector.grizzly.comet.CometContext;
import com.sun.enterprise.web.connector.grizzly.comet.CometEvent;
import com.sun.grizzly.cometd.bayeux.Connect;
import com.sun.grizzly.cometd.bayeux.Data;
import com.sun.grizzly.cometd.bayeux.Disconnect;
import com.sun.grizzly.cometd.bayeux.Handshake;
import com.sun.grizzly.cometd.bayeux.Reconnect;
import com.sun.grizzly.cometd.bayeux.Subscribe;
import com.sun.grizzly.cometd.bayeux.Unsubscribe;
import com.sun.grizzly.cometd.bayeux.Verb;
import java.io.IOException;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * This class implement the Bauyeux Server side protocol. 
 *
 * @author Jeanfrancois Arcand
 * @author TAKAI, Naot
 */
public class BayeuxCometHandler extends BayeuxCometHandlerBase{
    
    
    public final static String DEFAULT_CONTENT_TYPE ="application/json";
    
    
    public final static String BAYEUX_COMET_HANDLER = "bayeuxCometHandler";

    
    private ConcurrentLinkedQueue<String> clientIds 
            = new ConcurrentLinkedQueue<String>();
    
    
    private Random random = new Random();
    
    
    private ConcurrentHashMap<String,CometContext> channels =
            new ConcurrentHashMap<String,CometContext>();
    
    
    public void onHandshake(CometEvent event) throws IOException {            
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        CometdRequest req = cometdContext.getRequest();
        CometdResponse res = cometdContext.getResponse();
        Handshake handshake = (Handshake)cometdContext.getVerb();
        
        boolean handshakeOK = true;
        String clientId  = "";
        synchronized(random){
            clientId = String.valueOf(Long.toHexString(random.nextLong()));
        }
        clientIds.add(clientId);
        
        // XXX Why do we need to cache the ID. Memory leak right now
        if (handshakeOK){
            handshake.setClientId(clientId);
        }

        res.setContentType(DEFAULT_CONTENT_TYPE);            
        res.write(handshake.toJSON());
        res.flush();
    }
    
    
    public void onConnect(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        CometdRequest req = cometdContext.getRequest();
        CometdResponse res = cometdContext.getResponse();
        Connect connect = (Connect)cometdContext.getVerb();            
         
        res.setContentType(DEFAULT_CONTENT_TYPE);            
        res.write(connect.toJSON());
        res.flush();
    }
    
    
    public void onDisconnect(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        CometdRequest req = cometdContext.getRequest();
        CometdResponse res = cometdContext.getResponse();
        Disconnect disconnect = (Disconnect)cometdContext.getVerb();            
        
        res.setContentType(DEFAULT_CONTENT_TYPE);            
        res.write(disconnect.toJSON());
        res.flush();
    }
    

    public void onReconnect(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        CometdRequest req = cometdContext.getRequest();
        CometdResponse res = cometdContext.getResponse();
        Reconnect reconnect = (Reconnect)cometdContext.getVerb();                    
                       
        CometContext cometContext = event.getCometContext();
        DataHandler requestHandler = new DataHandler();
        requestHandler.attach(new Object[]{req,res});
        cometContext.addCometHandler(requestHandler);   
        
        res.setContentType(DEFAULT_CONTENT_TYPE);            
        res.write(reconnect.toJSON());
        res.flush();
    }
    
    
    public void onSubscribe(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        CometdRequest req = cometdContext.getRequest();
        CometdResponse res = cometdContext.getResponse();
        Subscribe subscribe = (Subscribe)cometdContext.getVerb();   
         
        res.setContentType(DEFAULT_CONTENT_TYPE);            
        res.write(subscribe.toJSON());
        res.flush();
    }
    
    
    public void onUnsubscribe(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        CometdRequest req = cometdContext.getRequest();
        CometdResponse res = cometdContext.getResponse();
        Unsubscribe unsubscribe = (Unsubscribe)cometdContext.getVerb();   
         
        res.setContentType(DEFAULT_CONTENT_TYPE);            
        res.write(unsubscribe.toJSON());
        res.flush();
    }
    
    
    public void onData(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        CometdRequest req = cometdContext.getRequest();
        CometdResponse res = cometdContext.getResponse();
        Data data = (Data)cometdContext.getVerb();           
         
        res.setContentType(DEFAULT_CONTENT_TYPE);            
        res.write(data.toJSON());
        res.flush();       
        event.getCometContext().notify(data);
    }
    
    
    public final static CometdContext newCometdContext(final CometdRequest req, 
            final CometdResponse res,final Verb verb){
        return new CometdContext(){
            
            public Verb getVerb(){
                return verb;
            }
            
            public CometdRequest getRequest(){
                return req;
            }
                      
            public CometdResponse getResponse(){
                return res;
            }            
        };
    }  
    
    // ---------------------------------------------------- Reserved but not used
    
    
    public void onPing(CometEvent event) throws IOException {
    }
    
    
    public void onStatus(CometEvent event) throws IOException {
    }
 
}