/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.grizzly.cometd.bayeux;

/**
 * Bayeux Advise implementation. 
 * See http://svn.xantus.org/shortbus/trunk/bayeux/protocol.txt for the technical
 * details.
 * 
 * The advice system provides a way for servers to inform clients of their
 * preferred mode of client operation. In conjunction with server-enforced limits,
 * Bayeux implementations can prevent resource exhaustion and inelegant failure in
 * several important edge cases. Since transport evenlopes are "pluggable" in
 * order to support differing "on the wire" behaviors, advice values may also vary
 * by transport type, even for the same conditions. What follows is a breif
 * description of the enumerations that are used in advices and their canonical
 * meanings.
 *
 * @author Jeanfrancois Arcand
 */
public class Advice extends VerbBase{
    
    // "none", "retry", "handshake", "recover"
    private String reconnect = "retry";
    
    
    private Integer interval = 0;
    
    
    private String transport = "\"long-polling\":\t{\n";
    
    
    public Advice() {
        type = Verb.ADVISE;
    }

    public String getReconnect() {
        return reconnect;
    }

    public void setReconnect(String reconnect) {
        this.reconnect = reconnect;
    }

    public Integer getInterval() {
        return interval;
    }

    public void setInterval(Integer interval) {
        this.interval = interval;
    }
        
    public String toString(){
        return toJSON();
    }
    
    public String toJSON() {        
        return "\"advice\":{" 
                + "\"reconnect\":\"" + reconnect + "\","
                + "\"interval\":" + interval + "}";         
    }

    public String getTransport() {
        return transport;
    }

    public void setTransport(String transport) {
        this.transport = transport;
    }
        
}
