/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.grizzly.cometd.bayeux;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Random;
import org.apache.tomcat.util.http.FastHttpDateFormat;

/**
 * Data representation of client/server interaction.
 *
 *	window.parent.cometd.deliver([
 *           {
 *                    // user-sent data
 *                    "data": {
 *                           "someField":	["some", "random", "values"],
 *                    },
 *                    // the usual message meta-data
 *                    "channel":		"/originating/channel",
 *                    // event ID
 *                    "id":			"slkjdlkj32",
 *                    "timestamp":	"TimeAtServer",
 *                    // optional meta-data
 *                    "authToken":	"SOME_NONCE_THAT_NEEDS_TO_BE_PROVIDED_SUBSEQUENTLY"
 *            },
 *            {
 *                    "data": {
 *                            "blah blah":	["more", "random", "values"],
 *                    },
 *                    // the usual message meta-data
 *                    "channel":		"/originating/channel",
 *                    // event ID
 *                    "id":			"slkjdlkj31",
 *                    "timestamp":	"TimeAtServer",
 *                    "authToken":	"SOME_NONCE_THAT_NEEDS_TO_BE_PROVIDED_SUBSEQUENTLY"
 *            }
 *            // , ...
 *    ]);
 *
 * @author Jeanfrancois Arcand
 */
public class Data extends VerbBase {
    
    private Random random = new Random();
    
    private HashMap<String,Object> data;

    private String connectionId;
    
    private String clientId;
    
    private String id;
    
    private String timestamp;
    
    public Data() {
        type = Verb.DATA;
    }

    public String toJSON() {        
        return "[{" 
                + "\"error\":\"" + error + "\","
                + "\"successful\":" + successful + ","                
                + "\"channel\":\"" + channel + "\""
                + "}]\n" ; 
    }

    public HashMap<String,Object> getData() {
        return data;
    }

    public void setData(HashMap<String,Object> data) {
        this.data = data;
    }
    
    public String getJSONData(){
        StringBuffer response = new StringBuffer();
        
        response.append("\n{");
        response.append("\"id\":\"");
        response.append(Long.toHexString(random.nextLong()));
        response.append("\",");
        response.append("\"timestamp\":\"");
        response.append(FastHttpDateFormat.getCurrentDate());
        response.append("\",");
        response.append("\"data\":{");        
        
        Iterator<String> iterator = data.keySet().iterator();
        String key = "";
        Object value;
        int size = data.size() -1;
        int i = 0;
        while(iterator.hasNext()){
            key = iterator.next();
            value = data.get(key);
            response.append("\"");
            response.append(key);
            response.append("\":");
            if (value instanceof String){
                response.append("\"" + value + "\"");   
            } else {
                response.append(value);  
            }
            
            if (i++ < size){
                response.append(",");
            }
        }
        response.append("},");  
        response.append("\"channel\":\"");
        response.append(channel);
        response.append("\"");
        response.append("}]");
        return response.toString();
    }

    public String getConnectionId() {
        return connectionId;
    }

    public void setConnectionId(String connectionId) {
        this.connectionId = connectionId;
    }

    public String getClientId() {
        return clientId;
    }

    public void setClientId(String clientId) {
        this.clientId = clientId;
    }
    
}
