/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.grizzly.cometd.bayeux;

import org.apache.tomcat.util.http.FastHttpDateFormat;

/**
 * Bayeux Reconnect implementation. 
 * See http://svn.xantus.org/shortbus/trunk/bayeux/protocol.txt for the technical
 * details.
 *
 * Upon connection, clients are implicitly subscribed to a connection-specific
 * channel located at:
 * 
 * 	/meta/clients/[SOME_UNIQUE_CLIENT_ID]
 * 
 * The client ID is used in reconnection. The reconnect verb allows occasionally
 * connected clients and the posting of messages for pre-authenticated systems.
 * 
 * 	// reconnection is very similar to initial connection:
 * 
 * 	//-----------------
 * 	// CLIENT -> SERVER
 * 	//-----------------
 * 
 * 	[
 * 		{
 * 			"channel":		"/meta/reconnect",
 * 			"clientId":		"SOME_UNIQUE_CLIENT_ID",
 * 			"timestamp":	"LastReceivedTimeAtServer",
 * 			"id":			"LastReceivedMessageId"
 * 			"connectionId":	"/meta/connections/26",
 * 			"connectionType": "iframe", // FIXME: is this necessaray?
 * 			// optional
 * 			"authToken":	"SOME_NONCE_PREVIOUSLY_PROVIDED_BY_SERVER"
 * 		}
 * 		// , ...
 * 	]
 * 
 * 
 * 	// generally, the first message in the array of responded messages will
 * 	// begin with:
 * 
 * 	//-----------------
 * 	// SERVER -> CLIENT
 * 	//-----------------
 * 
 * 	[
 * 		{
 * 			"channel":		"/meta/reconnect",
 * 			"connectionId":	"/meta/connections/26",
 * 			"successful":	true,
 * 			// optional
 * 			"authToken":	"SOME_NONCE_THAT_NEEDS_TO_BE_PROVIDED_SUBSEQUENTLY"
 * 		}
 * 		// , ...
 * 	]
 * @author Jeanfrancois Arcand
 */
public class Reconnect extends Connect{

    private Advice advise;
    
    public Reconnect() {
        type = Verb.RECONNECT;
    }

    public Advice getAdvise() {
        return advise;
    }

    public void setAdvise(Advice advise) {
        this.advise = advise;
    }
    
    public String toJSON() {            
        return "[{" 
                + "\"timestamp\":\"" + FastHttpDateFormat.getCurrentDate() + "\","
                + "\"connectionId\":\"" + META_CONNECTIONS + clientId + "\","                
                + "\"error\":\"" + error + "\","
                + "\"successful\":" + successful + ","                
                + "\"channel\":\"" + channel + "\""
                + "}," ;   
    }   
}
