/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.grizzly.cometd.bayeux;

/**
 * Bayeux Unsubscribe implementation. 
 * See http://svn.xantus.org/shortbus/trunk/bayeux/protocol.txt for the technical
 * details.
 *
 * // unsubscription for same:
 * 
 * 	//-----------------
 * 	// CLIENT -> SERVER
 * 	//-----------------
 * 
 * 	[
 * 		{
 * 			"channel":		"/meta/unsubscribe",
 * 			"subscription":	"/some/other/channel",
 * 			// optional
 * 			"authToken":	"SOME_NONCE_PREVIOUSLY_PROVIDED_BY_SERVER"
 * 		}
 * 		// , ...
 * 	]
 * 
 * 	// and the server response:
 * 
 * 	//-----------------
 * 	// SERVER -> CLIENT
 * 	//-----------------
 * 
 * 	[
 * 		{
 * 			"channel":		"/meta/unsubscribe",
 * 			"subscription":	"/some/other/channel",
 * 			"successful":	true,
 * 			"clientId":		"SOME_UNIQUE_CLIENT_ID",
 * 			"error":		"",
 * 			// optional
 * 			"authToken":	"SOME_NONCE"
 * 		}
 * 		// , ...
 * 	]
 * 	
 * @author Jeanfrancois Arcand
 */
public class Unsubscribe extends Subscribe{
    
    public Unsubscribe() {
        type = Verb.UNSUBSCRIBE;
    }

    public String toJSON() {        
        return "[{"              
                + "\"error\":\"" + error + "\","
                + "\"subscription\":\"" + subscription + "\"," 
                + "\"successful\":" + successful + ","                
                + "\"channel\":\"" + channel + "\""
                + "}]\n";
    }  
}
