/*
 * RedirectException.java
 * Created on February 16, 2007, 7:13 PM
 */

package com.sun.enterprise.admin.jmx.remote.comm;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class holds the redirect URL this HTTP JMX Connector client
 * receives when it tries to connect to http server with mismatching 
 * protocol/port/both on admin-listener.
 */
public class RedirectException extends IOException {
    
    private String redirectURL;
    private static final Logger logger = 
        Logger.getLogger(RedirectException.class.getName());
    private boolean debug = true;
    
    /** Creates a new instance of RedirectException */
    public RedirectException(String redirectURL) {
        this.redirectURL = redirectURL;
    }
    
    public URL getRedirectURL() {
        try {
            return new URL(redirectURL);
        } catch (MalformedURLException ex) {
            // log the exception as warning because most likely 
            // the subsequent attempt to redirect will fail
            logger.log(Level.WARNING, "Redirect URL received is malformed. " +
                                      "The URL is = " + redirectURL);
            if (debug) ex.printStackTrace();
        }
        return null;
    }
}